<?php
namespace Drupal\simple_account_policy;

use Drupal\simple_account_policy\Event\AccountPolicyActivateEvent;
use Drupal\simple_account_policy\Event\AccountPolicyBlockEvent;
use Drupal\simple_account_policy\Event\AccountPolicyDeleteEvent;
use Drupal\simple_account_policy\Event\AccountPolicyWarningEvent;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\UserInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

class AccountPolicy implements AccountPolicyInterface {

  use StringTranslationTrait;

  /** @var \Symfony\Component\EventDispatcher\EventDispatcherInterface */
  protected $event_dispatcher;

  /** @var \Drupal\Core\Database\Connection $db */
  protected $db;

  /** @var ConfigFactoryInterface $config_factory */
  protected $config_factory;

  /** @var StateInterface $state */
  protected $state;

  /** @var \Drupal\Core\Render\RendererInterface $renderer */
  protected $renderer;

  // Policy validation parameters.
  protected $config;

  /**
   * AccountPolicy constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Render\RendererInterface $renderer
   */
  public function __construct(ConfigFactoryInterface $config_factory, Connection $database, StateInterface $state, EventDispatcherInterface $eventDispatcher, RendererInterface $renderer) {
    $this->config_factory = $config_factory;
    $this->db = $database;
    $this->state = $state;
    $this->event_dispatcher = $eventDispatcher;
    $this->renderer = $renderer;

    $this->config = $this->config_factory->getEditable('simple_account_policy.settings');
  }

  /**
   * @inheritDoc
   */
  public function applyPolicy(UserInterface $user) {

    $ignore_user = false;

    $username_ignore_patterns = $this->config->get('username_ignore_patterns') ?? [];
    if (!empty($username_ignore_patterns)) {
      $pattern = '(' . implode('|', array_map('preg_quote', array_filter($username_ignore_patterns))) . ')';
      $ignore_user = (preg_match("/$pattern/", $user->getAccountName(), $matches));
    }

    return $user
      // Don't apply if user has bypass permission
      && !$user->hasPermission("bypass account policy")
      // Don't apply if user need to be ignored (by account name)
      && !$ignore_user
      // Dont apply for anonymous users.
      && !$user->isAnonymous();
  }

  /**
   * @inheritDoc
   */
  public function validate(UserInterface $user, $data = []) {
    $errors = [];

    if ($this->applyPolicy($user)) {

      // Get the values to validate. Use data first if provided.
      $mail = $data['mail'] ?? $user->getEmail();
      $username = $data['name'] ?? $user->getAccountName();

      // Check if username matches email.
      $username_match_email = $this->config->get('username_match_email') ?? false;
      if ($username_match_email !== false) {
        if ($mail !== $username) {
          $errors['name'][] = 'username_match_email';
        }
      }

      // Check if username matches patterns.
      $username_match_patterns = $this->config->get('username_match_patterns') ?? [];
      if (!empty($username_match_patterns)) {
        foreach ($username_match_patterns as $delta => $pattern) {
          if (!preg_match("/$pattern/", $mail, $matches)) {
            $errors['name'][] = 'username_match_pattern_' . $delta;
          }
        }
      }

      // Check if email matches patterns.
      $email_match_patterns = $this->config->get('email_match_patterns') ?? [];
      if (!empty($email_match_patterns)) {
        foreach ($email_match_patterns as $delta => $pattern) {
          if (!preg_match("/$pattern/", $mail, $matches)) {
            $errors['mail'][] = 'email_match_pattern_' . $delta;
          }
        }
      }
    }

    return $errors;
  }

  /**
   * @inheritDoc
   */
  public function policy(UserInterface $user = null, array $errors = []) {

    // Make sure we have the default field keys.
    $errors += [
      'mail' => [],
      'name' => []
    ];

    $policy = [];

    if ($this->applyPolicy($user)) {

      // Check if username matches email.
      $username_match_email = $this->config->get('username_match_email') ?? false;
      if ($username_match_email !== false) {
        $policy['name'][] = [
          '#wrapper_attributes' => ['class' => [in_array('username_match_email', $errors['name']) ? 'account-policy-invalid-rule marker' : 'account-policy-valid-rule']],
          '#markup' => (string) $this->t("The username must match the email address.")
        ];
      }

      // Check if username matches patterns.
      $username_match_patterns = $this->config->get('username_match_patterns') ?? [];
      if (!empty($username_match_patterns)) {
        foreach ($username_match_patterns as $delta => $pattern) {
          $policy['name'][] = [
            '#wrapper_attributes' => ['class' => [in_array('username_match_pattern_' . $delta, $errors['name']) ? 'account-policy-invalid-rule marker' : 'account-policy-valid-rule']],
            '#markup' => (string) $this->patternToMessage($pattern)
          ];
        }
      }

      // Check if email matches patterns.
      $email_match_patterns = $this->config->get('email_match_patterns') ?? [];
      if (!empty($email_match_patterns)) {
        foreach ($email_match_patterns as $delta => $pattern) {
          $policy['mail'][] = [
            '#wrapper_attributes' => ['class' => [in_array('email_match_pattern_' . $delta, $errors['name']) ? 'account-policy-invalid-rule marker' : 'account-policy-valid-rule']],
            '#markup' => (string) $this->patternToMessage($pattern)
          ];
        }
      }

      if (!empty($policy['name'])) {
        $elements = [
          'title' => [
            '#markup' => (string) $this->formatPlural(count($policy['name']),
              "The username must satisfy the following account policy rule:",
              "The username must satisfy the following account policy rules:"
            )
          ],
          'rules' => [
            '#theme' => 'item_list',
            '#list_type' => 'ul',
            '#items' => $policy['name']
          ]
        ];
        $policy['name'] = $this->renderer->renderPlain($elements);
      }

      if (!empty($policy['mail'])) {
        $elements = [
          'title' => [
            '#markup' => (string) $this->formatPlural(count($policy['mail']),
              "The email must satisfy the following account policy rule:",
              "The email must satisfy the following account policy rules:"
            )
          ],
          'rules' => [
            '#theme' => 'item_list',
            '#list_type' => 'ul',
            '#items' => $policy['mail']
          ]
        ];
        $policy['mail'] = $this->renderer->renderPlain($elements);
      }
    }
    else {
      $policy = [
        'mail' => $this->t("This user bypasses the account policy."),
        'name' => $this->t("This user bypasses the account policy."),
      ];
    }

    return $policy;
  }

  /**
   * @inerhitDoc
   */
  public function getBlockTime(UserInterface $user) {
    $block_time = 0;

    $inactive_period = (string) $this->config->get('inactive_period') ?? '';

    if (!empty($inactive_period)) {

      $lastAccessedTime = $user->getLastAccessedTime();
      if ($lastAccessedTime === '0') {
        $lastAccessedTime = $user->getCreatedTime();
      }

      if (ctype_digit($inactive_period)) {
        $block_time = $lastAccessedTime + intval($inactive_period);
      }
      else {
        $block_time = time() + ($lastAccessedTime - strtotime('-' . $inactive_period));
      }
    }

    return $block_time;
  }

  /**
   * @inerhitDoc
   */
  public function getWarningTime(UserInterface $user) {
    $warning_time = 0;

    $block_time = $this->getBlockTime($user);
    if ($block_time) {

      $inactive_warning = (string) $this->config->get('inactive_warning') ?? '';
      if (!empty($inactive_warning)) {

        if (ctype_digit($inactive_warning)) {
          $warning_time = $block_time - intval($inactive_warning);
        }
        else {
          $warning_time = time() + ($block_time - strtotime('+' . $inactive_warning));
        }

      }
    }

    return $warning_time;
  }

  /**
   * @inerhitDoc
   */
  public function inactiveInterval() {
    return $this->config->get('inactive_interval') ?? 86400;
  }

  /**
   * @inerhitDoc
   */
  public function getDeleteAfterTime() {

    $delete_after_time = (string) $this->config->get('delete_after_time') ?? 0;

    if (!empty($delete_after_time)) {

      if (ctype_digit($delete_after_time)) {
        $delete_after_time = time() - intval($delete_after_time);
      }
      else {
        $delete_after_time = strtotime("$delete_after_time ago") ?: 0;
      }

    }

    return $delete_after_time;
  }

  /**
   * @inheritDoc
   */
  public function shouldIssueWarning(UserInterface $user) {
    $warning_time = $this->getWarningTime($user);
    return ( $warning_time && !$this->warningIssued($user) && time() > $warning_time );
  }

  /**
   * @inheritDoc
   */
  public function warningIssued(UserInterface $user) {
    $warned_users = $this->state->get('simple_account_policy.warned_users', []);
    return key_exists($user->id(), $warned_users);
  }

  /**
   * @inheritDoc
   */
  public function issueWarning(UserInterface $user) {
    // Keep track of issued warnings.
    $warned_users = $this->state->get('simple_account_policy.warned_users', []);
    $warned_users[$user->id()] = $user->id();
    $this->state->set('simple_account_policy.warned_users', $warned_users);

    // Send out an event so others can react to this as well.
    $event = new AccountPolicyWarningEvent($user, $this);
    $this->event_dispatcher->dispatch($event, AccountPolicyWarningEvent::EVENT_NAME);
  }

  public function isInactive(UserInterface $user) {

    $block_time = $this->getBlockTime($user);

    return ($block_time && $user->isActive() && time() > $block_time);
  }

  /**
   * @inerhitDoc
   */
  public function block(UserInterface $user) {
    $event = new AccountPolicyBlockEvent($user, $this);
    $this->event_dispatcher->dispatch($event, AccountPolicyBlockEvent::EVENT_NAME);
  }

  /**
   * @inerhitDoc
   */
  public function activate(UserInterface $user) {
    // Keep track of issued warnings.
    $warned_users = $this->state->get('simple_account_policy.warned_users', []);
    unset($warned_users[$user->id()]);
    $this->state->set('simple_account_policy.warned_users', $warned_users);

    // Send out an event so others can react to this as well.
    $event = new AccountPolicyActivateEvent($user, $this);
    $this->event_dispatcher->dispatch($event, AccountPolicyActivateEvent::EVENT_NAME);
  }

  /**
   * Pretty print a regular expression pattern.
   *
   * @param $pattern
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  protected function patternToMessage($pattern) {

    $beginsWithApostrophe = $pattern[0] == '^';
    $endsWithDollar =  $pattern[strlen($pattern)-1] == '$';

    if ($beginsWithApostrophe && $endsWithDollar) {
      $message = $this->t("Must match %pattern.", ['%pattern' => trim($pattern, "^$")]);
    }
    else {
      if ($beginsWithApostrophe) {
        $message = $this->t("Must begin with %pattern.", ['%pattern' => ltrim($pattern, "^")]);
      } else if ($endsWithDollar) {
        $message = $this->t("Must end with %pattern.", ['%pattern' => rtrim($pattern, "$")]);
      }
      else {
        $message = $this->t("Must contain %pattern.", ['%pattern' => $pattern]);
      }
    }

    return $message;
  }

  /**
   * @inheritdoc
   */
  public function shouldBeDeleted(UserInterface $user) {
    $lastLoginTime = $user->getLastAccessedTime();
    $deleteAfterTime = $this->getDeleteAfterTime();
    if (empty($deleteAfterTime)) {
      return FALSE;
    }

    if ($lastLoginTime < $deleteAfterTime) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * @inheritdoc
   */
  public function delete(UserInterface $user) {
    $cancelMethod = $this->config->get('user_cancel_method');

    if (empty($cancelMethod)) {
      $cancelMethod = 'user_cancel_reassign';
    }

    $event = new AccountPolicyDeleteEvent($user, $this, $cancelMethod);
    $this->event_dispatcher->dispatch($event, AccountPolicyDeleteEvent::EVENT_NAME);
  }

}
