<?php
namespace Drupal\simple_account_policy;

use Drupal\user\UserInterface;

interface AccountPolicyInterface {

  /**
   * Decide if the policy should be applied for this user or not.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function applyPolicy(UserInterface $user);

  /**
   * Validate an email string for a given user.
   *
   * @param \Drupal\user\UserInterface $user
   * @param array $data
   *   An array containing the data to validate for the given user.
   *   Should contain a 'mail' and 'name' key.
   *
   * @return array
   *   An array of policy violations (errors) for 'mail' and 'name'.
   */
  public function validate(UserInterface $user, $data);

  /**
   * Return the policy that applies for the given user.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return array
   *  Array containing a string describing the policy for 'mail' and 'name'.
   */
  public function policy(UserInterface $user);

  /**
   * Get the timestamp an inactive user will get blocked.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return int
   */
  public function getBlockTime(UserInterface $user);

  /**
   * Get the timestamp to warn an inactive user will get blocked.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return int
   */
  public function getWarningTime(UserInterface $user);

  /**
   * Get the timestamp when a user can be removed.
   *
   * @return int
   */
  public function getDeleteAfterTime();

  /**
   * The interval in seconds to check for inactive users.
   *
   * @return int
   */
  public function inactiveInterval();

  /**
   * Decide if we need to issue a warning or not.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function shouldIssueWarning(UserInterface $user);

  /**
   * Decide if we need to delete a user or not.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function shouldBeDeleted(UserInterface $user);

  /**
   * Check if a user already got a warning.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function warningIssued(UserInterface $user);

  /**
   * Send out a warning event.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function issueWarning(UserInterface $user);

  /**
   * Decide if user is considered active or not.
   *
   * @param \Drupal\user\UserInterface $user
   */
  public function isInactive(UserInterface $user);

  /**
   * Block a given user.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return mixed
   */
  public function block(UserInterface $user);

  /**
   * Delete a given user.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return mixed
   */
  public function delete(UserInterface $user);

  /**
   * Make a given user active again.
   *
   * The user will be reactivated and his last login date will be reset to today
   * to prevent him being blocked again by cron.
   *
   * @param \Drupal\user\UserInterface $user
   *
   * @return mixed
   */
  public function activate(UserInterface $user);

}
